==> aidlab_sdk-0.0.15.tar.gz: android/src/main/kotlin/com/aidlab/aidlab_sdk/AidlabSdkHandler.kt <==
package com.aidlab.aidlab_sdk

import android.os.Handler
import android.os.Looper
import com.aidlab.sdk.Logger
import com.aidlab.sdk.communication.Signal
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.BinaryMessenger
import java.lang.IllegalArgumentException
import java.util.*

class AidlabSdkHandler: AidlabFlutter.PlatformAidlabSdk {

    var aidlabSynchronizationHandler: AidlabSynchronizationHandler? = null
    var aidlabHandler: AidlabHandler? = null
    
    private var aidlabSdkListener: AidlabFlutter.PlatformAidlabSdkDelegate? = null
    private var aidlabLoggerListener: AidlabFlutter.PlatformAidlabLogger? = null
    
    private lateinit var binaryMessenger: BinaryMessenger

    fun onAttachedToEngine(binding: FlutterPlugin.FlutterPluginBinding) {

        AidlabBluetoothManager.init(binding.applicationContext)
        binaryMessenger = binding.binaryMessenger

        aidlabSdkListener = AidlabFlutter.PlatformAidlabSdkDelegate(binding.binaryMessenger)
        AidlabFlutter.PlatformAidlabSdk.setup(binding.binaryMessenger, this)

        aidlabLoggerListener = AidlabFlutter.PlatformAidlabLogger(binding.binaryMessenger)

        Logger.listener = object: Logger.Listener {
            override fun onError(msg: String, throwable: Throwable) {
                Handler(Looper.getMainLooper()).post {
                    aidlabLoggerListener?.onError(
                        AidlabFlutter.LogEvent().apply { this.msg = "$msg - $throwable" }) {}
                }
            }

            override fun onLog(msg: String) {
                Handler(Looper.getMainLooper()).post {
                    aidlabLoggerListener?.onLog(
                        AidlabFlutter.LogEvent().apply { this.msg = msg }) {}
                }
            }

            override fun onWarning(msg: String) {
                Handler(Looper.getMainLooper()).post {
                    aidlabLoggerListener?.onWarning(
                        AidlabFlutter.LogEvent().apply { this.msg = msg }) {}
                }
            }

        }
    }

    override fun init() {

        AidlabBluetoothManager.lastConnectedAidlab?.disconnect()
        
        release()

        AidlabBluetoothManager.listener = aidlabSdkListener
        aidlabSynchronizationHandler = AidlabSynchronizationHandler(binaryMessenger)
        aidlabHandler = AidlabHandler(aidlabSynchronizationHandler!!, binaryMessenger)
    }


    fun release() {
        AidlabBluetoothManager.stopScan()
        AidlabBluetoothManager.listener = null

        aidlabHandler?.release()
        aidlabSynchronizationHandler?.release()
    }
    
    override fun scanForDevices(arg: AidlabFlutter.ScanModeRequest?) {
        //TODO implement mode
        AidlabBluetoothManager.startScan()
    }

    override fun stopDeviceScan() {
        AidlabBluetoothManager.stopScan()
    }

    override fun connect(arg: AidlabFlutter.ConnectRequest?) {
        AidlabBluetoothManager.stopScan()
        val signals = arg!!.signals.asIterable()
                .map { signalValue ->
                    Signal.values().asIterable().find {
                        it.value == signalValue
                    } ?: throw IllegalArgumentException("Failed to parse signal with value: $signalValue")
                }

        AidlabBluetoothManager.connect(
                arg.address,
                EnumSet.copyOf(signals),
                aidlabHandler!!
        )
    }
}

==> aidlab_sdk-0.2.0.tar.gz: android/src/main/kotlin/com/aidlab/aidlab_sdk/AidlabSdkHandler.kt <==
package com.aidlab.aidlab_sdk

import android.os.Handler
import android.os.Looper
import com.aidlab.sdk.Logger
import com.aidlab.sdk.communication.Signal
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.BinaryMessenger
import java.lang.IllegalArgumentException
import java.util.*

class AidlabSdkHandler: PlatformAidlabSdk {

    var aidlabSynchronizationHandler: AidlabSynchronizationHandler? = null
    var aidlabHandler: AidlabHandler? = null
    
    private var aidlabSdkListener: PlatformAidlabSdkDelegate? = null
    private var aidlabLoggerListener: PlatformAidlabLogger? = null
    
    private lateinit var binaryMessenger: BinaryMessenger

    fun onAttachedToEngine(binding: FlutterPlugin.FlutterPluginBinding) {

        AidlabBluetoothManager.init(binding.applicationContext)
        binaryMessenger = binding.binaryMessenger

        aidlabSdkListener = PlatformAidlabSdkDelegate(binding.binaryMessenger)
        PlatformAidlabSdk.setUp(binding.binaryMessenger, this)

        aidlabLoggerListener = PlatformAidlabLogger(binding.binaryMessenger)

        Logger.listener = object: Logger.Listener {
            override fun onError(msg: String, throwable: Throwable) {
                Handler(Looper.getMainLooper()).post {
                    aidlabLoggerListener?.onError(LogEvent("$msg - $throwable")) {}
                }
            }

            override fun onLog(msg: String) {
                Handler(Looper.getMainLooper()).post {
                    aidlabLoggerListener?.onLog(LogEvent(msg)) {}
                }
            }

            override fun onWarning(msg: String) {
                Handler(Looper.getMainLooper()).post {
                    aidlabLoggerListener?.onWarning(LogEvent(msg)) {}
                }
            }
        }
    }

    override fun initSdk() {

        AidlabBluetoothManager.lastConnectedAidlab?.disconnect()
        
        release()

        AidlabBluetoothManager.listener = aidlabSdkListener
        aidlabSynchronizationHandler = AidlabSynchronizationHandler(binaryMessenger)
        aidlabHandler = AidlabHandler(aidlabSynchronizationHandler!!, binaryMessenger)
    }


    fun release() {
        AidlabBluetoothManager.stopScan()
        AidlabBluetoothManager.listener = null

        aidlabHandler?.release()
        aidlabSynchronizationHandler?.release()
    }
    
    override fun scanForDevices(scanMode: ScanModeRequest) {
        //TODO implement mode
        AidlabBluetoothManager.startScan()
    }

    override fun stopDeviceScan() {
        AidlabBluetoothManager.stopScan()
    }

    override fun connect(request: ConnectRequest) {
        AidlabBluetoothManager.stopScan()
        val signals = request.signals?.asIterable()?.map { signalValue ->
                Signal.values().asIterable().find {
                    it.value == signalValue
                } ?: throw IllegalArgumentException("Failed to parse signal with value: $signalValue")
            }

        AidlabBluetoothManager.connect(
            request.address!!,
            EnumSet.copyOf(signals),
            aidlabHandler!!
        )
    }
}
